#include <algorithm>
#include <iostream>
#include <string>
#include <list>
#include <stdexcept>
#include "PackageTree.h"
#include "Package.h"
#include "Interaction.h"
#include "String.h"
#include "Mapping.h"
using namespace std;

// mapuje nazwe grupy na inta, a potem przypisuje nazwe pakietu do grupy
void PackageTree::set_mapping(const string& grp, const string& pkg)
{
	map<string, int>::iterator it;
	int g;
	
	it=Group::mapping.find(grp);
	
	if(it==Group::mapping.end())
	{
		g=Group::name.size();
		Group::mapping[grp]=g;
		Group::name.push_back(grp);
	}
	else
		g=it->second;
	
	pkg2grp[pkg]=g;
}

void PackageTree::clear_installed_status(TreeDir& td)
{
	list<TreeDir>& grp=td.get_groups();
	list<Package>& pkg=td.get_packages();
	list<Package>::iterator it;

	for_each(grp.begin(), grp.end(), PackageTree::clear_installed_status);
	for(it=pkg.begin(); it!=pkg.end(); it++)
		it->setinstalled(false);
}

PackageTree::PackageTree()
	:root("ROOT")
{}

PackageTree::~PackageTree()
{}

int PackageTree::fillup()
{
	int count=0;
	list<string> cmd;
	string s;
	
	cmd.push_back("poldek");
	cmd.push_back("-q");
	cmd.push_back("--fast");
	cmd.push_back("--shcmd=desc *");
	
	Interaction i(cmd);
	
	s=i.gets();
	while(!i.eof())
	{
		// nastpny pakiet
		if(s.substr(0,8)=="Package:")
		{
			Package p;
			string group;
			list<string> nvr, dsc;
			list<string>::const_iterator iter;
			
			// pobieranie odpowiednich pl
			// name-version-release
			String::split_nvr(s.substr(16), back_inserter(nvr));
			iter=nvr.begin();
			p.setname(*iter++);
			p.setversion(*iter++);
			p.setrelease(String::chop(*iter));
			
			// summary
			s=i.gets();
			p.setsummary(String::chop(s.substr(16)));
			
			// group
			s=i.gets();
			group=String::chop(s.substr(16));
			
			// vendor - opuci
			i.gets();
			
			// license
			s=i.gets();
			p.setlicense(String::chop(s.substr(16)));
			
			// arch/os - opuci
			i.gets();
			
			// url - moe nie by zdefiniowane
			s=i.gets();
			if(s.substr(0,4)=="URL:")
			{
				p.seturl(String::chop(s.substr(16)));
				
				s=i.gets();
			}
			
			// built - opuci, cig ju jest pobrany
			
			// size
			s=i.gets();
			p.setsize(String::extract_size(s.substr(16)));
			
			// package size
			s=i.gets();
			p.setpkgsize(String::extract_size(s.substr(16)));
			
			// path - opuci
			s=i.gets();

			// moliwe epoch
			s=i.gets();

			// description (moliwe e ju pobrane przed chwil)
			if(s.substr(0, 6)=="Epoch:")
				i.gets();

			for(;;)
			{
				s=i.gets();
				if(s.substr(0,8)=="Package:" || i.eof())
					break;
				
				dsc.push_back(s);
			}
			p.setdescription(dsc);

			count++;
			
			root.add(group, p);
			
			set_mapping(group, p.getname());
		}
		else
			s=i.gets();
	}

	if(i.terminate())
		throw runtime_error("poldek exited abnormally");
	
	return count;
}

int PackageTree::check_installed()
{
	int count=0;
	list<string> cmd;
	string s;
	
	cmd.push_back("rpm");
	cmd.push_back("-qa");
	cmd.push_back("--qf=%{name}\\n");
	
	Interaction i(cmd);
	
	s=i.gets();
	while(!i.eof())
	{
		try
		{
			Package& p=find_package(String::chop(s));
			p.setinstalled(true);;
			count++;
		}
		catch(invalid_argument){};
		// zignorowa brak pakietu - mona mie pakiety spoza
		// dystrybucji

		s=i.gets();
	}

	return count;
}

int PackageTree::recheck_installed()
{
	clear_installed_status(root);
	return check_installed();
}

void PackageTree::clear()
{
	root.clear();
}

TreeDir& PackageTree::find_group(const string& group)
{
	return root.find_group(group);
}

Package& PackageTree::find_package(const string& pkg)
{
	map<string, int>::iterator it;

	it=pkg2grp.find(pkg);

	if(it==pkg2grp.end())
		throw invalid_argument("no such package");
	
	TreeDir& td=find_group(Group::name[it->second]);
	
	return td.find_package(pkg);
}

TreeDir& PackageTree::root_group()
{
	return root;
}
