#include <assert.h>
#include <stdexcept>
#include <string>
#include <list>
#include <iterator>
#include "Transaction.h"
#include "Interaction.h"
#include "String.h"
using namespace std;

string Transaction::poldek_error;

void Transaction::add_install(const string& s)
{
	ins.push_back(s);
}

void Transaction::add_dep(const string& s)
{
	dep.push_back(s);
}

void Transaction::add_remove(const string& s)
{
	rem.push_back(s);
}

void Transaction::add_error(const string& s)
{
	err.push_back(s);
}

Transaction::Transaction()
	: ready(false), working_interaction(NULL)
{}

Transaction::~Transaction()
{
	if(working_interaction)
		delete working_interaction;
}

const list<string>& Transaction::get_installs() const
{
	return ins;
}

const list<string>& Transaction::get_deps() const
{
	return dep;
}

const list<string>& Transaction::get_removes() const
{
	return rem;
}

const list<string>& Transaction::get_errors() const
{
	return err;
}

void Transaction::check_poldek_error()
{
	list<string> cmd;

	cmd.push_back("poldek");
	cmd.push_back("-nTransaction::check_poldek_error()-dummy-set");

	Interaction i(cmd);

	string s=i.gets();	// bd: bleble: nie ma takiego rda

	poldek_error=String::extract_error(s);
}

void Transaction::do_install()
{
	list<string> cmd;

	cmd.push_back("poldek");
	cmd.push_back("-i");

	list<string>::const_iterator i;

	for(i=ins.begin(); i!=ins.end(); i++)
	{
		string::size_type j;

		j=i->rfind('-');
		j=i->rfind('-', j-1);
		
		cmd.push_back(i->substr(0, j));
	}
	for(i=dep.begin(); i!=dep.end(); i++)
	{
		string::size_type j;

		j=i->rfind('-');
		j=i->rfind('-', j-1);
		
		cmd.push_back(i->substr(0, j));
	}

	working_interaction=new Interaction(cmd);
}

void Transaction::do_uninstall()
{
	list<string> cmd;

	cmd.push_back("poldek");
	cmd.push_back("-e");
	cmd.push_back("--noask");

	list<string>::const_iterator i;

	for(i=rem.begin(); i!=rem.end(); i++)
	{
		string::size_type j;

		j=i->rfind('-');
		j=i->rfind('-', j-1);
		
		cmd.push_back(i->substr(0, j));
	}
	for(i=dep.begin(); i!=dep.end(); i++)
	{
		string::size_type j;

		j=i->rfind('-');
		j=i->rfind('-', j-1);
		
		cmd.push_back(i->substr(0, j));
	}

	working_interaction=new Interaction(cmd);
}

bool Transaction::try_command(const string& p, int op)
{
	list<string> cmd;
	string s;

	assert(op==OP_INSTALL || op==OP_UNINSTALL);

	if(poldek_error.empty())
		check_poldek_error();

	ins.clear();
	dep.clear();
	rem.clear();
	err.clear();

	command=op;

	cmd.push_back("poldek");
	if(op==OP_INSTALL)
	{
		cmd.push_back("-iv");
		cmd.push_back("--test");
		cmd.push_back("--noask");
	}
	else
		cmd.push_back("-e");
	cmd.push_back(p);
	
	Interaction i(cmd);

	// poldek si pyta czy rzeczywicie usun (jak sie stdin nie zamknie)
//	if(op==OP_UNINSTALL)
//		i.puts("n\n");

	while(!i.eof())
	{
		s=String::chop(i.gets());
		
		if(s.substr(0, poldek_error.size())==poldek_error)
			add_error(s.substr(poldek_error.size()));
		else if(s.substr(0, 2)=="I ")
			String::ungroup(s.substr(2), back_inserter(ins));
		else if(s.substr(0, 2)=="D ")
			String::ungroup(s.substr(2), back_inserter(dep));
		else if(s.substr(0, 2)=="R ")
			String::ungroup(s.substr(2), back_inserter(rem));
	}

	if(err.empty())
		ready=true;

	return err.empty();
}

void Transaction::doit()
{
	start();
	while(!working_interaction->eof())
		working_interaction->gets();
}

void Transaction::start()
{
	if(!ready)
		throw runtime_error("transaction not ready");
	if(working_interaction)
		throw runtime_error("transaction already working");

	if(command==OP_INSTALL)
		do_install();
	else
		do_uninstall();
}

int Transaction::getc()
{
	return working_interaction->getc();
}

string Transaction::gets()
{
	return working_interaction->gets();
}

bool Transaction::eof()
{
	return working_interaction->eof();
}
